using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Web;
using System.Web.Compilation;
using System.Web.Mvc;
using System.Web.Optimization;
using System.Web.Routing;
using FluentValidation.Mvc;
using HIPS.Web.Components.Web;
using HIPS.Web.UI.Controllers;
using MvcAuthorization.Policy;
using System.Globalization;
using System.Web.Script.Serialization;
using System.IO;
using System.Collections;

namespace HIPS.Web.UI
{
    // Note: For instructions on enabling IIS6 or IIS7 classic mode,
    // visit http://go.microsoft.com/?LinkId=9394801
    [Authorize]
    public class MvcApplication : HttpApplication
    {
        public MvcApplication()
        {
            HIPS.Web.UI.Helpers.MixedAuthExtensions.RegisterWindowsAuthentication(this);
        }

        /// <summary>
        /// Builds a list of policy handler types in the current AppDomain and retrieves their metadata.
        /// </summary>
        /// <returns></returns>
        internal static List<Tuple<Type, PolicyMetadataAttribute>> LoadAndCachePolicyHandlers()
        {
            List<Tuple<Type, PolicyMetadataAttribute>> result = new List<Tuple<Type, PolicyMetadataAttribute>>();

            // Changed to BuildManager.GetReferencedAssemblies() here
            var policyHandlerTypes = BuildManager.GetReferencedAssemblies().Cast<Assembly>().SelectMany(x => x.GetTypes())
                                                    .Where(t => typeof(IAuthorizationPolicy).IsAssignableFrom(t) && !t.IsAbstract && t.IsClass && t.IsPublic && !t.IsGenericType);

            result = policyHandlerTypes
                .Select(x => new Tuple<Type, PolicyMetadataAttribute>(x, (PolicyMetadataAttribute)Attribute.GetCustomAttribute(x, typeof(PolicyMetadataAttribute)))).ToList();

            // Find empty handlers
            var emptyPolicyHandlers = result.Where(x => x.Item2 == null || string.IsNullOrWhiteSpace(x.Item2.Name));

            if (emptyPolicyHandlers.Count() > 0)
            {
                throw new ApplicationException(string.Format("Policy names are required. The following policy type(s) inherit from IAuthorizationPolicy but do not have a policy name set through the PolicyMetadataAttribute: {0}", string.Join(", ", emptyPolicyHandlers.Select(g => g.Item1.FullName))));
            }

            // Find duplicates and throw an error if there is more than one policy handler with the same name
            var duplicateHandlers = result.Where(x => x.Item2 != null).Select(x => x.Item2).GroupBy(x => x.Name).Where(x => x.Count() > 1);

            if (duplicateHandlers.Count() > 0)
            {
                throw new ApplicationException(string.Format("Policy names must be unique. The following policy name(s) were found more than once: {0}", string.Join(", ", duplicateHandlers.Select(g => g.Key))));
            }

            return result;
        }

        protected void Application_Start()
        {
            typeof(MvcAuthorization.Policy.AuthorizationPolicy).Assembly
                .GetType("MvcAuthorization.Policy.PolicyHelper")
                .GetField("_policyHandlerTypeListCache", BindingFlags.Static | BindingFlags.NonPublic)
                .SetValue(null,
                    new Lazy<List<Tuple<Type, PolicyMetadataAttribute>>>(() => LoadAndCachePolicyHandlers()));
            // Register Areas (Currently Unused)
            AreaRegistration.RegisterAllAreas();

            // Register Routes
            RouteConfig.RegisterRoutes(RouteTable.Routes);

            // Register Javascript + CSS Bundles
            BundleConfig.RegisterBundles(BundleTable.Bundles);

            // Register Validators
            RegisterValidatorProviders();

            // Register Model Binders
            RegisterModelBinders();

            ViewEngines.Engines.Add(new RazorViewEngine());

            // This block increases the max json request length
            // --------------------------------------------------------------------------
            JsonValueProviderFactory jsonValueProviderFactory = null;

            foreach (var factory in ValueProviderFactories.Factories)
            {
                if (factory is JsonValueProviderFactory)
                {
                    jsonValueProviderFactory = factory as JsonValueProviderFactory;
                }
            }

            // Remove the default JsonVAlueProviderFactory
            if (jsonValueProviderFactory != null) ValueProviderFactories.Factories.Remove(jsonValueProviderFactory);

            // Add the custom one
            ValueProviderFactories.Factories.Add(new CustomJsonValueProviderFactory());
            // ------------------------------------------------------------------------------
        }

        public static void RegisterModelBinders()
        {
            ModelBinders.Binders.Add(typeof(DateTime), new CultureAwareDateBinder());
            ModelBinders.Binders.Add(typeof(DateTime?), new CultureAwareNullableDateBinder());
        }

        public virtual void RegisterValidatorProviders()
        {
            // Register FluentValidation (TODO: Verify if needed for nested data annotations)
            FluentValidationModelValidatorProvider.Configure();
        }

        protected void Application_EndRequest()
        {
            // TODO: A better way to address error handling responses and still allow 422 or others as passthrough.
            if (Response.StatusCode != 422)
            {
                // Setting status to itself triggers internal response behaviour
                // This allows Exceptions to redirect via httpErrors element to the error controller
                //Response.Status = Response.Status;
            }
        }

        /// <summary>
        /// On Application Error check the Exception and send to the exception class if required.
        /// </summary>
        protected void Application_Error()
        {
            var exception = Server.GetLastError();
            var httpException = exception as HttpException;
            Response.Clear();
            Server.ClearError();
            var routeData = new RouteData();
            routeData.Values["controller"] = "Error";
            routeData.Values["action"] = "Index"; //default
            routeData.Values["exception"] = exception;
            //Response.StatusCode = 500;

            if (httpException != null)
            {
                // Log details:
                Elmah.ErrorSignal.FromCurrentContext().Raise(httpException);
                switch (httpException.GetType().Name)
                {
                    case "HttpRequestValidationException":

                        routeData.Values["action"] = "HttpRequestValidationException";
                        break;
                }
            }
            else if (exception != null)
            {
                // Log details:
                Elmah.ErrorSignal.FromCurrentContext().Raise(exception);
                switch (exception.GetType().Name)
                {
                    case "EndpointNotFoundException":

                        routeData.Values["action"] = "EndpointNotFoundException";
                        break;
                }
            }
            else
            {
                // Log details:
                Elmah.ErrorSignal.FromCurrentContext().Raise(new Exception("An unhandled exception has occurred."));
            }

            IController errorController = new ErrorController();
            var rc = new RequestContext(new HttpContextWrapper(Context), routeData);
            errorController.Execute(rc);
        }

        /// <summary>
        /// Hide sensitive data in Elmah logs
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void ErrorLog_Filtering(object sender, Elmah.ExceptionFilterEventArgs e)
        {
            var ctx = e.Context as HttpContext;
            if (ctx == null)
            {
                return;
            }
            HIPS.Web.UI.Helpers.ElmahExtensions.ApplySensitiveDataFilter(e, ctx);
        }
    }

    public sealed class CustomJsonValueProviderFactory : ValueProviderFactory
    {

        private static void AddToBackingStore(Dictionary<string, object> backingStore, string prefix, object value)
        {
            IDictionary<string, object> d = value as IDictionary<string, object>;
            if (d != null)
            {
                foreach (KeyValuePair<string, object> entry in d)
                {
                    AddToBackingStore(backingStore, MakePropertyKey(prefix, entry.Key), entry.Value);
                }
                return;
            }

            IList l = value as IList;
            if (l != null)
            {
                for (int i = 0; i < l.Count; i++)
                {
                    AddToBackingStore(backingStore, MakeArrayKey(prefix, i), l[i]);
                }
                return;
            }

            // primitive
            backingStore[prefix] = value;
        }

        private static object GetDeserializedObject(ControllerContext controllerContext)
        {
            if (!controllerContext.HttpContext.Request.ContentType.StartsWith("application/json", StringComparison.OrdinalIgnoreCase))
            {
                // not JSON request
                return null;
            }

            StreamReader reader = new StreamReader(controllerContext.HttpContext.Request.InputStream);
            string bodyText = reader.ReadToEnd();
            if (string.IsNullOrEmpty(bodyText))
            {
                // no JSON data
                return null;
            }

            JavaScriptSerializer serializer = new JavaScriptSerializer();
            serializer.MaxJsonLength = int.MaxValue; //increase MaxJsonLength.  This could be read in from the web.config if you prefer
            object jsonData = serializer.DeserializeObject(bodyText);
            return jsonData;
        }

        public override IValueProvider GetValueProvider(ControllerContext controllerContext)
        {
            if (controllerContext == null)
            {
                throw new ArgumentNullException("controllerContext");
            }

            object jsonData = GetDeserializedObject(controllerContext);
            if (jsonData == null)
            {
                return null;
            }

            Dictionary<string, object> backingStore = new Dictionary<string, object>(StringComparer.OrdinalIgnoreCase);
            AddToBackingStore(backingStore, string.Empty, jsonData);
            return new DictionaryValueProvider<object>(backingStore, CultureInfo.CurrentCulture);
        }

        private static string MakeArrayKey(string prefix, int index)
        {
            return prefix + "[" + index.ToString(CultureInfo.InvariantCulture) + "]";
        }

        private static string MakePropertyKey(string prefix, string propertyName)
        {
            return (string.IsNullOrEmpty(prefix)) ? propertyName : prefix + "." + propertyName;
        }
    }
}